;; Generated package description from company.el
(define-package "company" "0.6.14" "Modular in-buffer completion framework" 'nil :url "http://company-mode.github.io/" :keywords '("abbrev" "convenience" "matching"))
